/***************************************************************************************** 
*                                     TinyRtos51 
*                                The Real-Time Kernel 
*                              Copyright(c) 2024 fanxsp
*                         SPDX-License-Identifier: APACHE-2.0
*       This software is subject to an open source license and is distributed by fanxsp,
*       pursuant to the terms of the Apache License.
******************************************************************************************/
/******************************************************************************************* 
* 				main.c文件
* Author: fanxsp 
* Revision: 1.00
* Date: 2023-05-03
* 	1. 在tinyrtos51cfg.h 中配置 OS
*   2. 在port.h port_xxx.h中适配不同型号的 MCU
*   3. 在option的c51中的预定义宏中定义目标MCU型号
*   4. 如果使能定时服务，需要在一个定时器中调用OSTimeTick()函数，定时中断应设为最低优先级
********************************************************************************************/

/*************  功能说明    ****************************************************************
* 本例程基于STC8H8K64U核心实验板（开天斧）进行编写测试。
* 用逻辑分析仪测量任务切换时间
********************************************************************************************/
#include "tinyRtos51.h"

#define MAIN_Fosc       24000000L   
#define Timer0_Reload   (65536UL -(MAIN_Fosc / 1000))       //Timer0 中断频率 1000次/秒

/* 任务定义 -------------------------------------------------------------------------------*/
/* 任务堆栈  */
OStypeInt8u OSLOC_STACKS taskAStack[30] ;
OStypeInt8u OSLOC_STACKS taskBStack[30] ;
OStypeInt8u OSLOC_STACKS taskCStack[30] ;
/* 任务ID  */
#define ID_TASKA    0
#define ID_TASKB    1
#define ID_TASKC    2
/* 任务优先级 */
#define PRIO_TASKA  4
#define PRIO_TASKB  5
#define PRIO_TASKC  6
/* 任务函数原型 */
void TaskA(void) ;
void TaskB(void) ;
void TaskC(void) ;
/*-----------------------------------------------------------------------------------------*/

void Init(void) ;

void main(void)
{
/* OS初始化  */
	OSInit() ;
	
/* 创建任务、二值信号量、计数信号量、事件标志、消息队列  放在这里 */
	OSCreateTask(ID_TASKA,TaskA,taskAStack,PRIO_TASKA) ;
	OSCreateTask(ID_TASKB,TaskB,taskBStack,PRIO_TASKB) ;
	OSCreateTask(ID_TASKC,TaskC,taskCStack,PRIO_TASKC) ;

/*----------------------------------------------------------------------------------------*/
	
/* 用户初始化 放在这里  */
	Init() ;
/*----------------------------------------------------------------------------------------*/
	
/* 启动OS */ 
	OSStart() ;
}

/* 任务A 测试被中断唤醒时间  */
void TaskA(void)
{
	int i ;

//====== 无限循环 ===========================	
	for(;;)
	{
//		T3L = 0x40;		   //设置定时初始值
//		T3H = 0xA2;		   //设置定时初始值
//		T4T3M |= 0x08;	   //定时器3开始计时
		OSTaskSuspend() ;  // 任务挂起
//------ 任务被中断唤醒 ---------------------
		P01 = 0 ;
//		T4T3M &= ~(0x08) ; //定时器3停止计时
//------ 中断事件处理开始 -------------------
		P02 = 1 ;
		for(i=0;i<30;i++)
		{
			P02 = ~P02 ;
			OSDelay(5) ;
		}
//------- 中断事件处理结束 -------------------
	}
}

/* 任务B 测试被任务唤醒时间 */
void TaskB(void)
{
//====== 无限循环 ===========================	
	for(;;)
	{
		OSTaskSuspend() ;  // 任务挂起
//------ 任务被程序启动 ---------------------
		P03 = 0 ;
//------ 任务处理开始 -----------------------
		P04 = 1 ;
		OSDelay(50) ;
		P04 = 0 ;
//------ 任务处理结束 -----------------------
	}
}

void TaskC(void)
{
	int i ;
	
//====== 无限循环 ===========================	
	for(;;)
	{
		for(i=0;i<30;i++)
		{
			P05 = ~P05 ;
			OSDelay(5) ;
		}
//------ 启动高优先级任务 -----------------------
		P03 = 1 ;
		OSTaskResume(ID_TASKB) ;
	}
}

void ISR_Timer0(void) interrupt 1  // 1ms 中断函数
{
	static unsigned long Timer0Cnt ;
	
	Timer0Cnt ++ ;
	P06 = ~P06 ;
	
//	OSTimeTick() ;    // OS节拍处理
}

void ISR_Timer3(void) interrupt 19  
{
	static unsigned char i ;
	
	OSTimeTick() ;    // OS节拍处理
	
	if(++i > 199)
	{
		i = 0 ;
		P00 = ~P00 ;
		P01 = 1 ;
		OSIntResume(ID_TASKA) ;  // 间隔200ms 恢复任务A
	}
}

void Init(void)
{
	P_SW2 |= 0x80;     //扩展寄存器(XFR)访问使能
	P0M1 = 0x00;   P0M0 = 0x00;   //设置为准双向口
	P1M1 = 0x00;   P1M0 = 0x00;   //设置为准双向口
	P2M1 = 0x00;   P2M0 = 0x00;   //设置为准双向口
	P3M1 = 0x00;   P3M0 = 0x00;   //设置为准双向口
	P4M1 = 0x00;   P4M0 = 0x00;   //设置为准双向口
	P5M1 = 0x00;   P5M0 = 0x00;   //设置为准双向口
	P6M1 = 0x00;   P6M0 = 0x00;   //设置为准双向口
	P7M1 = 0x00;   P7M0 = 0x00;   //设置为准双向口
	
	/* 1000微秒@24.000MHz */
	T4T3M |= 0x02;			//定时器时钟1T模式
	T3L = 0x40;				//设置定时初始值
	T3H = 0xA2;				//设置定时初始值
	IE2 |= 0x20;			//使能定时器3中断

/*	
	AUXR = 0x80;            //Timer0 1T, 16 bits 自动重装载模式 
	TMOD = 0x03 ;           //Timer0 不可屏蔽中断的16位重装载模式
	TH0 = (unsigned char)(Timer0_Reload / 256);
	TL0 = (unsigned char)(Timer0_Reload % 256);
	ET0 = 1;                //Timer0 中断使能
	TR0 = 1;                //Timer0 开始计时
*/
	/* 2us @22.1184M */
	AUXR |= 0x80;			//定时器时钟1T模式
	TMOD = 0x03;			//设置定时器模式 非屏蔽16位重装载
	TL0 = 0xD4;				//设置定时初始值
	TH0 = 0xFF;				//设置定时初始值
	ET0 = 1;                //Timer0 中断使能
	TR0 = 1;				//定时器0开始计时

	T4T3M |= 0x08;			//定时器3开始计时
}

