/***************************************************************************************** 
*                                     TinyRtos51 
*                                The Real-Time Kernel 
*                              Copyright(c) 2024 fanxsp
*                         SPDX-License-Identifier: APACHE-2.0
*       This software is subject to an open source license and is distributed by fanxsp,
*       pursuant to the terms of the Apache License.
******************************************************************************************/
/***************************************************************************************** 
* 				TinyRtos51 OSMsg.c
* Author: fanxsp 
* Revision: 1.00
* Date: 2024-04-30
* 
******************************************************************************************/
#define __IN_OSMSG_C
#include "tinyrtos51.h"

#if OSMSGS > 0

/* MsgQ 控制块 */
OStypeInt8u OSLOC_MSG OSMsgArea[OSMSGS] ; // 0-没有消息

/* API */
/*****************************************************************************************
* 初始化消息  在OSStart()前调用
* 参数：eID: 消息ID   value:消息初始值
******************************************************************************************/
void OSInitMsg(OStypeInt8u eID,OStypeInt8u value) 
{
#if OSENABLE_ERROR_CHECKING > 0
	if(eID > OSMSGS-1) return ;
#endif
	OSMsgArea[eID] = value  ;
}

/******************************************************************************************
* 等待消息  在任务中调用 不能在中断中调用
* 参数：eID: 消息ID  msg: 消息指针,用于接收消息,发生超时则消息指针指向的数据是不可预知的
*      ticks: 超时时间，=0 为永久等待
*******************************************************************************************/
#if OSBYTES_OF_DELAYS > 0
void OSWaitMsg(OStypeInt8u eID,OStypeInt8u idata *msg,OStypeOfDelays ticks) 
#else
void OSWaitMsg(OStypeInt8u eID,OStypeInt8u idata *msg) 	
#endif
{
#if OSENABLE_ERROR_CHECKING > 0
	if(eID > OSMSGS-1) return ;
#endif
	OSEnterCritical() ;
	OSTcbArea[OSTaskRunning].msg = OSMsgArea[eID] ;
	OSMsgArea[eID] = 0 ;
#if OSBYTES_OF_DELAYS > 0
	OSTcbArea[OSTaskRunning].timeout = 0 ;
	if((OSTcbArea[OSTaskRunning].msg == 0)&&(OSTcbArea[OSTaskRunning].timeout == 0))
#else
	if(OSTcbArea[OSTaskRunning].msg == 0)
#endif
	{
		OSTcbArea[OSTaskRunning].state = OSTCB_WAITING ;
#if OSBYTES_OF_DELAYS > 0
		OSTcbArea[OSTaskRunning].delay = ticks ;
#endif
		ACC = (OStypeInt8u)msg ;
		_push_(ACC) ;
		ACC = eID ;
		_push_(ACC) ;
		OSDelFromList(OSTaskRunning) ;
		eID = *(OStypeInt8u idata *)SP ;
		OSAppendToList(OSHeadMsg(eID),OSTaskRunning) ;
		OSTaskSw() ;
		_pop_(ACC) ;
		eID = ACC ;
		_pop_(ACC) ;
		msg = (OStypeInt8u idata *)ACC ;
	}
	
#if OSBYTES_OF_DELAYS > 0
	if(OSTcbArea[OSTaskRunning].timeout == 0)
	{
#endif
		*msg = OSTcbArea[OSTaskRunning].msg ;
#if OSBYTES_OF_DELAYS > 0
	}
#endif
	OSExitCritical() ;
}

/**********************************************************************************************
* 测试并清除消息 不进入等待状态 可以在中断中调用
* 参数：eID: 消息ID
* 返回：消息值,0-表示没有消息
**********************************************************************************************/
OStypeInt8u OSTryMsg(OStypeInt8u eID) 
{
	OStypeInt8u tmp ;
	
#if OSENABLE_ERROR_CHECKING > 0
	if(eID > OSMSGQS-1) return 0 ;
#endif
	OSEnterCritical() ;
	tmp = OSMsgArea[eID] ;
	OSMsgArea[eID] = 0 ;
	OSExitCritical() ;
	return tmp ;
}

/**********************************************************************************************
* 读取但不清除消息、不进入等待状态，可以在中断中调用
* 参数：eID: 消息ID 
* 返回：消息值,0-表示没有消息 
**********************************************************************************************/
OStypeInt8u OSReadMsg(OStypeInt8u eID) 
{
#if OSENABLE_ERROR_CHECKING > 0
	if(eID > OSMSGQS-1) return 0 ;
#endif
	return OSMsgArea[eID] ;
}

/************************************************************************************************
* 发送消息 公共代码 不含临界保护 不能直接调有 
* 参数：eID: 消息队列ID  msg: 消息 0-表示没有消息
*************************************************************************************************/
void OSSignalMsg(OStypeInt8u eID,OStypeInt8u msg) 
{
	OStypeInt8u tmp ;

	if(OSHeadMsg(eID) != OSListArea[OSHeadMsg(eID)].next)
	{
		tmp = OSHiPrioInList(OSHeadMsg(eID)) ;
		OSTcbArea[tmp].state = OSTCB_ELIGIBLE ;
		OSTcbArea[tmp].msg = msg ;
#if OSBYTES_OF_DELAYS > 0
		OSTcbArea[tmp].delay = 0 ;
#endif
//		SP++ ;
//		*(OStypeInt8u idata *)SP = tmp ;
		OSDelFromList(tmp) ;
//		tmp = *(OStypeInt8u idata *)SP ;
//		SP-- ;
		OSAppendToList(OSHeadEligible,tmp) ;
	}
	else OSMsgArea[eID] = msg ;
}

/************************************************************************************************
* 在任务中发送消息  
* 参数：eID: 消息队列ID  msg: 消息 0-表示没有消息
*************************************************************************************************/
void OSTaskSignalMsg(OStypeInt8u eID,OStypeInt8u msg) 
{
#if OSENABLE_ERROR_CHECKING > 0
	if(eID > OSMSGQS-1) return ;
	if(msg == 0) return ;
#endif

	OSEnterCritical() ;
	OSSignalMsg(eID,msg) ;
#if OSMETHOD > 0
	OSTaskSw() ;
#endif
	OSExitCritical() ;
}

/************************************************************************************************
* 在中断中发送消息
* 参数：eID: 消息队列ID  msg: 消息 0-表示没有消息
************************************************************************************************/
void OSIntSignalMsg(OStypeInt8u eID,OStypeInt8u msg) 
{
#if OSENABLE_ERROR_CHECKING > 0
	if(eID > OSMSGS-1) return ;
	if(msg == 0) return ;
#endif

	OSEnterCritical() ;
	OSSignalMsg(eID,msg) ;
#if OSMETHOD > 0
	OSPendSVAct() ;
#endif
	OSExitCritical() ;
}

#endif

